use io;

/* Read instance data */
function input() {
    local usage = "Usage: localsolver flowshop.lsp "
            + "inFileName=inputFile [lsTimeLimit=timeLimit]";

    if (inFileName == nil) throw usage;
    local inFile = io.openRead(inFileName);

    nbJobs = inFile.readInt();
    nbMachines = inFile.readInt();
    inFile.readInt();
    inFile.readInt();
    inFile.readInt();
    processingTime[m in 0...nbMachines][j in 0...nbJobs] = inFile.readInt();
}

/* Declare the optimization model */
function model() {
    // Permutation of jobs
    jobs <- list(nbJobs);

    // All jobs have to be assigned
    constraint count(jobs) == nbJobs;

    // On machine 0, the jth job ends on the time it took to be processed after 
    // the end of the previous job
    jobEnd[0] <- array(0...nbJobs, (i, prev) => prev + processingTime[0][jobs[i]], 0);

    // The jth job on machine m starts when it has been processed by machine n-1
    // AND when job j-1 has been processed on machine m.
    // It ends after it has been processed.
    for [m in 1...nbMachines]
        jobEnd[m] <- array(0...nbJobs,
                (i, prev) => max(prev, jobEnd[m-1][i]) + processingTime[m][jobs[i]], 0);

    // Minimize the makespan: end of the last job on the last machine
    makespan <- jobEnd[nbMachines-1][nbJobs-1];
    minimize makespan;
}

/* Parametrize the solver */
function param() {
    if (lsTimeLimit == nil) lsTimeLimit = 5;
}

/* Write the solution in a file */
function output() {
    if (solFileName == nil) return;

    local solFile = io.openWrite(solFileName);
    solFile.println(makespan.value);
    for [j in jobs.value]
        solFile.print(j + " ");
    solFile.println();
}
